#!/usr/bin/env python

# Inspire by Conbagui tutorial found here: https://www.deviantart.com/conbagui/art/Create-a-custom-Lace-brush-and-Text-effect-853378041

# author: Pocholo
# date: 10/1/20

# Comments directed to http://gimpchat.com


# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.


from gimpfu import *

def pm_lace_text_effect(customtext, fontname, textcolor, fontsize, justify, 
                                                                   linespa, 
															   backpattern, 
																 lacecolor):
	
	
	
	size = 250
	
	#Create the lace brush
	lacebrush = pdb.gimp_brush_new("Lace brush")
	shape_out = pdb.gimp_brush_set_shape("Lace brush", BRUSH_GENERATED_SQUARE)
	radius_out = pdb.gimp_brush_set_radius("Lace brush", 123.3)
	spikes_out = pdb.gimp_brush_set_spikes("Lace brush", 12)
	hardness_out = pdb.gimp_brush_set_hardness("Lace brush", 1)
	aspect_ratio_out = pdb.gimp_brush_set_aspect_ratio("Lace brush", 1.0)
	angle_out = pdb.gimp_brush_set_angle("Lace brush", 0)
	pdb.gimp_brush_set_spacing("Lace brush", 49)
	
	
	#New image for the brush	
	img = pdb.gimp_image_new(size, size, RGB_IMAGE)
	lace = pdb.gimp_layer_new(img, size, size, RGB, "Lace brush", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, lace, 0)
	pdb.gimp_layer_add_alpha(lace)
	pdb.gimp_drawable_fill(lace, FILL_TRANSPARENT)
		
	#Show process
	pdb.gimp_display_new(img)
	
	#Make the designs on the brush
	pdb.gimp_context_set_foreground((0, 0, 0))
	pdb.gimp_brushes_set_brush("Lace brush")
	pdb.gimp_paintbrush_default(lace, 2, [125, 125])
	pdb.gimp_image_select_ellipse(img, 2, 25, 25, 200, 200)
	pdb.plug_in_sel2path(img, lace)
	pdb.gimp_selection_none(img)
	#Create the big holes
	pdb.gimp_brushes_set_brush("2. Hardness 100")
	pdb.gimp_context_set_brush_size(20)
	pdb.gimp_context_set_brush_aspect_ratio(0)
	pdb.gimp_context_set_brush_angle(0)
	pdb.gimp_context_set_brush_spacing(2.30)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_paint_method("gimp-eraser")
	pdb.gimp_context_set_stroke_method(STROKE_PAINT_METHOD)
	pdb.gimp_context_set_dynamics('Dynamics Off')
	vectors = pdb.gimp_image_get_active_vectors(img)
	pdb.gimp_edit_stroke_vectors(lace, vectors)
	#create the small holes
	pdb.gimp_image_select_item(img, 2, vectors)
	pdb.gimp_selection_shrink(img, 25)
	pdb.plug_in_sel2path(img, lace)
	pdb.gimp_selection_none(img)
	pdb.gimp_brushes_set_brush("2. Hardness 100")
	pdb.gimp_context_set_brush_size(10)
	pdb.gimp_context_set_brush_aspect_ratio(0)
	pdb.gimp_context_set_brush_angle(0)
	pdb.gimp_context_set_brush_spacing(2.00)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_paint_method("gimp-eraser")
	pdb.gimp_context_set_stroke_method(STROKE_PAINT_METHOD)
	pdb.gimp_context_set_dynamics('Dynamics Off')
	vectors = pdb.gimp_image_get_active_vectors(img)
	pdb.gimp_edit_stroke_vectors(lace, vectors)
		
	#Initiates the temporary state
	pdb.gimp_context_push()
	
	#Copy brush
	non_empty = pdb.gimp_edit_copy(lace)
		
	#Create the lace text effect
	
	#Variables
	width = 1600
	height = 1200
	
	#Create a new image
	img = pdb.gimp_image_new(1, 1, RGB)
	
	#Variables
	width = img.width
	height = img.height
	
	#Create the text
	pdb.gimp_context_set_foreground(textcolor)
	textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 50, TRUE, fontsize, PIXELS, fontname)
	pdb.gimp_text_layer_set_line_spacing(textLayer, linespa)
	pdb.gimp_text_layer_set_justification(textLayer, justify)
	
	#Resize image
	img.resize(textLayer.width, textLayer.height, 0, 0)
	
	#Create background
	backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGB_IMAGE, "Rose pattern", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, backLayer, 1)
	pdb.gimp_context_set_pattern(backpattern)
	pdb.gimp_edit_bucket_fill(backLayer, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
		
	#Show process
	pdb.gimp_display_new(img)
	
	pdb.gimp_image_select_item(img, 2, textLayer)
	pdb.plug_in_sel2path(img, textLayer)
	pdb.gimp_selection_invert(img)
	
	#Create the lace layer
	lace = pdb.gimp_layer_new(img, img.width, img.height, RGB, "Lace", 100, LAYER_MODE_NORMAL)
	pdb.gimp_image_add_layer(img, lace, 1)
	pdb.gimp_layer_add_alpha(lace)
	pdb.gimp_drawable_fill(lace, FILL_TRANSPARENT)
	
	#Create the lace around the text
	pdb.gimp_context_set_foreground(lacecolor)
	pdb.gimp_brushes_set_brush("Clipboard Image")
	pdb.gimp_context_set_brush_size(35)
	pdb.gimp_context_set_brush_spacing(0.69)
	pdb.gimp_context_set_brush_default_hardness()
	pdb.gimp_context_set_paint_method("gimp-paintbrush")
	pdb.gimp_context_set_stroke_method(STROKE_PAINT_METHOD)
	pdb.gimp_drawable_edit_stroke_selection(lace)
	pdb.gimp_drawable_invert(lace, TRUE)
	
	#Delete the brush
	pdb.gimp_brush_delete("Lace brush")
	pdb.gimp_selection_none(img)
	
	#Bevel and emboss
	pdb.python_layerfx_bevel_emboss(img, textLayer, 
												  1,   #style inner bevel
												 65,   #depth 
												  0,   #direction 
												 10,   #size 
												 10,   #soften
												120,   #angle 
												 30,   #altitude 
												  3,   #gloss_contour- linear 
								    (255, 255, 255),   #highlight_color 
												  4,   #highlight_mode - screen
												 80,   #highlight_opacity
										  (0, 0, 0),   #shadow_color 
												  3,   #shadow_mode - multiply
											 	 25,   #shadow_opacity 
												  0,   #surface_contour - linear 
											  FALSE,   #use_texture 
									   "Dried mud",   #pattern 
											     50,   #scale 
											    100,   #tex_depth 
											   TRUE,   #invert 
											   FALSE)   #merge
	
	
	#Create a new visible layer at the top of the pile
	pdb.gimp_item_set_visible(backLayer, FALSE)
	laceText = pdb.gimp_layer_new_from_visible(img, img, "Lace Text")
	pdb.gimp_image_add_layer(img, laceText, 0)
	
	#Drop Shadow
	pdb.python_layerfx_drop_shadow(img, laceText,
								         (0, 0, 0),   #color
											    75,   #opacity
												 0,   #contou - linear
												 0,   #noise
												 3,   #mode - multiply   
												 0,   #spread
											     4,   #size
											   120,   #offset_angle
												10,   #offset_distance
											  TRUE,   #knockout
											 FALSE)   #merge
	
	pdb.gimp_item_set_visible(backLayer, TRUE)
	
	#Resets the previous user defaults
	pdb.gimp_context_pop()
	
register(
	"pm_lace_text_effect",
	"Creates an inward repetition",
	"Creates an image inward repetition",
	"Pocholo",
	"Pocholo",
	"2020",
	"Lace Text Effect",
	"",
[
	(PF_TEXT, "customtext", "Text string", 'Love\nAlways'),
    (PF_FONT, "font", "Font face", "ChopinScript"),	
	(PF_COLOR, "textcolor", "Text color", (248, 19, 24)),
	(PF_ADJUSTMENT, "fontsize", "Size", 300, (300, 450, 1)),
	(PF_OPTION, "justify", "Justify", 2, ("Left", "Right", "Centered", "Fill")),
	(PF_ADJUSTMENT, "linespa", "Line spacing", -50, (-100, 100, 1)),
	(PF_PATTERN, "backpattern", "Background pattern", "Rose pattern fabric"),
	(PF_COLOR, "lacecolor", "Lace color", (255, 255, 255)),
],
[],
pm_lace_text_effect, menu="<Image>/Pocholo-scripts/Lace Text Effect")
main()							